" VIM syntax script for 64tass, a 65xx crossassembler
"
" Prefixes of classes start with 'tass64', since Vim doesn't allow
" indentifiers to start with '64tass', obviously.
"
" author:   Bas Wassink (Compyx/Focus) <b.wassink@ziggo.nl>
" version:  0.1 (2016-03-28)

syn clear
syn case ignore

" Registers
syn keyword tass64Reg   b d k r s x y z

" 6502 Mnemonics
syn keyword tass64Mne   adc and asl
syn keyword tass64Mne   bcc bcs beq bit bmi bne bpl brek bvc
syn keyword tass64Mne   clc cld cli clv cmp cpx cpy
syn keyword tass64Mne   dec dex dey
syn keyword tass64Mne   eor
syn keyword tass64Mne   inc inx iny
syn keyword tass64Mne   jmp jsr
syn keyword tass64Mne   lda ldx ldy lsr
syn keyword tass64Mne   nop
syn keyword tass64Mne   ora
syn keyword tass64Mne   pha php pla plp
syn keyword tass64Mne   rol ror rti rts
syn keyword tass64Mne   sbc sec sed sei sta stx sty
syn keyword tass64Mne   tax tay tsx txa txs tya

" 6502 Illegal opcodes
syn keyword tass64MneIll    ahx anc ane arr asr axs
syn keyword tass64MneIll    dcm dcp
syn keyword tass64MneIll    ins isb isc
syn keyword tass64MneIll    jam
syn keyword tass64MneIll    lae las lax lds lxa
syn keyword tass64MneIll    ral rra
syn keyword tass64MneIll    sax sbx sha shs shx shy slo sre
syn keyword tass64MneIll    tas
syn keyword tass64MneIll    xaa

" Assembler directives
" Some regexes needed here to handle the leading dot
syn match tass64Dir /\.a\(l\(ign\)\?\|s\(sert\)\?\)/
syn match tass64Dir /\.b\(end\|include\|lock\|reak\)/
syn match tass64Dir /\.c\(ase\|def\|error\|heck\|o\(mment\|ntinue\)\|pu\|warn\)/
syn match tass64Dir /\.d\(atabank\|efault\|page\|section\|struct\|union\)/
syn match tass64Dir /\.e\(def\|ls\(e\|if\)\|n\(c\|d\(c\|f\|if\|m\|p\|s\(witch\)\?\|u\|weak\)\?\)\|or\|rror\)/
syn match tass64Dir /\.f\(i\|or\|unction\)/
syn match tass64Dir /\.goto/
syn match tass64Dir /\.here/
syn match tass64Dir /\.hidemac/
syn match tass64Dir /\.i\(f\(eq\|mi\|ne\|pl\)\?\|nclude\)/
syn match tass64Dir /\.l\(bl\|ogical\)/
syn match tass64Dir /\.macro/
syn match tass64Dir /\.next/
syn match tass64Dir /\.o\(ffs\|ption\)/
syn match tass64Dir /\.p\(age\|end\|ro\(c\|ff\|n\)\)/
syn match tass64Dir /\.rept/
syn match tass64Dir /\.s\(e\(ction\|ed\|gment\|nd\)\|howmac\|truct\|witch\)/
syn match tass64Dir /\.union/
syn match tass64Dir /.var/
syn match tass64Dir /\.w\(arn\|eak\)/
syn match tass64Dir /\.x[ls]/

" Data statements
syn match tass64Data    /\.addr/
syn match tass64Data    /\.b\(inary\|yte\)/
syn match tass64Data    /\.char/
syn match tass64Data    /\.d\(int\|word\)/
syn match tass64Data    /\.fill/
syn match tass64Data    /\.int/
syn match tass64Data    /\.l\(int\|ong\)/
syn match tass64Data    /\.null/
syn match tass64Data    /\.ptext/
syn match tass64Data    /\.rta/
syn match tass64Data    /\.shift[l]\?/
syn match tass64Data    /\.text/
syn match tass64Data    /\.word/


" Built-in functions
syn keyword tass64Func  abs acos all any asin atan atan2
syn keyword tass64Func  cbrt ceil cos cosh
syn keyword tass64Func  deg
syn keyword tass64Func  exp
syn keyword tass64Func  floor format frac
syn keyword tass64Func  hypot
syn keyword tass64Func  len log log10
syn keyword tass64Func  pow
syn keyword tass64Func  rad random range repr round
syn keyword tass64Func  sign sin sinh size sqrt
syn keyword tass64Func  tan tanh trunc

" Built-in types
syn keyword tass64Type  address
syn keyword tass64Type  bits bool bytes
syn keyword tass64Type  code
syn keyword tass64Type  dict
syn keyword tass64Type  float
syn keyword tass64Type  gap
syn keyword tass64Type  int
syn keyword tass64Type  list
syn keyword tass64Type  str
syn keyword tass64Type  tuple
syn keyword tass64Type  type


" Macro references
syn match tass64Macro   '^\s*#[a-zA-Z_][a-zA-Z0-9_]*'

" Labels
syn match tass64Label   '\<[a-zA-Z_][a-zA-Z0-9_]*\>'

" Comments
syn match tass64Comment ';.*$' contains=tass64Todo
syn keyword tass64Todo  contained  fixme todo note xxx

" Numeric constants
syn match tass64Bin     '\<%[01]\+\>'
syn match tass64Dec     '\<\d\+\>'
syn match tass64Hex     '\$[0-9a-fA-F]\+\>'

" Strings
syn region tass64String start="\"" skip="\\\"" end="\""

" Program counter reference/assignment
syn match tass64PC    '\*\s*\[=\+=]'

" Color setup
hi link tass64Mne       Statement
hi link tass64MneIll    Statement
hi link tass64Reg       Statement
hi link tass64Dir       PreProc
hi link tass64Data      Special
hi link tass64Comment   Comment
hi link tass64Bin       Number
hi link tass64Dec       Number
hi link tass64Hex       Number
hi link tass64String    String
hi link tass64Type      Type
hi link tass64Func      Function
hi link tass64Macro     Function
" hi link tass64Label     Identifier
hi link tass64PC        Special

